//+=============================================================================
//
// file :         ABending.cpp
//
// description :  C++ source for the ABending and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                ABending are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.14 $
//
// $Revision: 1.14 $
// $Date: 2019-01-14 16:12:42 $
//
// SVN only:
// $HeadURL: $
//
// CVS only:
// $Source: /home/cvsadm/cvsroot/fermi/servers/bending/src/ABending.cpp,v $
// $Log: ABending.cpp,v $
// Revision 1.14  2019-01-14 16:12:42  claudio
// #6723: 0 field when OFF,FAULT,STANDBY
//
// Revision 1.13  2018-08-28 13:32:12  claudio
// TANGO_BASE_CLASS, #4939: 0 field if OFF/FAULT/STANDBY
//
// Revision 1.12  2017-08-17 12:43:27  claudio
// #1554 tango-9.2.5a
//
// Revision 1.11  2012-06-05 13:11:00  claudio
// added ExcitationSetPoint attribute
//
// Revision 1.10  2012-06-04 15:48:24  claudio
// read e setting separati
// piu' robusta gestione eccezzioni
//
// Revision 1.9  2012-02-29 13:34:40  claudio
// bug in init
//
// Revision 1.8  2012-02-29 13:00:42  claudio
// ticket 891:  aggiunti attributi per leggere minimi e massimi delle grandezze impostabili. Vengono ricalcolati quando cambia l'energia.
//
// Revision 1.7  2012-01-31 11:30:45  claudio
// ticket 876: problema di eccezioni mal gestite. risolto
// aggiunta update anche in always_executed()
// messo a fattor comune il metodo di update
//
// Revision 1.6  2011/04/26 11:58:32  claudio
// delete magnet factory object soon after usage.
// Do to stanger mysql error on exit of prgroams: exit ins not immediat, delayed by some 10 seconds and
// and complains about mysql_tread not closed. This error appeared when started using mysql 5.5
//
// Revision 1.5  2010/11/17 14:10:21  claudio
// remeoved unused vars.
// starts also with power supply devices down
//
// Revision 1.4  2010/09/10 09:34:10  claudio
// handle also power supply (anomalous) with DevFloat attribute
// (see ticket 356)
//
// Revision 1.3  2010/09/02 14:14:34  claudio
// improved handling of device state:
// - mirrors power supply state
// - fault if cannto read power supply
// - is OFF or STANDBY force 0 reading and setting
// - rationalized handling of update values
//
// Revision 1.2  2010/03/31 15:38:08  claudio
// added Kick in milliradians
//
// Revision 1.1  2010/02/23 15:45:19  claudio
// bending magnet model - first import
//
//
// copyleft :     Sincrotrone Trieste S.C.p.A
//                Strada Statale 14 - km 163,5 in AREA Science Park
//                34149 Basovizza, Trieste ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name|  Method name
//	----------------------------------------
//  State   |  dev_state()
//  Status  |  dev_status()
//
//===================================================================


#include <tango.h>
#include <ABending.h>
#include <ABendingClass.h>
#include <dipole.h>

namespace ABending_ns
{

//+----------------------------------------------------------------------------
//
// method : 		ABending::ABending(std::string &s)
// 
// description : 	constructor for simulated ABending
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
ABending::ABending(Tango::DeviceClass *cl,std::string &s)
	:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

ABending::ABending(Tango::DeviceClass *cl,const char *s)
	:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

ABending::ABending(Tango::DeviceClass *cl,const char *s,const char *d)
	:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		ABending::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void ABending::delete_device()
{
	//	Delete device allocated objects
	delete dev;
	if(magnet)
		delete magnet;
// 	if(mf)
// 		delete mf;
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void ABending::init_device()
{
	INFO_STREAM << "ABending::ABending() create device " << device_name << std::endl;
	
	// Initialise variables to default values
	/* Allocate the read attributes */
	attr_MagnetEnergy_read = &magnet_energy_read;
	attr_MagnetEnergySetPoint_read = &magnet_energy_set_point;
	attr_Excitation_read=&excitation_read;
	attr_ExcitationSetPoint_read =&excitation_set_point;
	attr_Energy_read=&energy;
	attr_Kick_read=&kick_read;
	attr_KickSetPoint_read=&kick_set_point;
        
        
        attr_MinMagnetEnergy_read = &min_me;
        attr_MaxMagnetEnergy_read = &max_me;
        attr_MinKick_read = &min_kick;
        attr_MaxKick_read = &max_kick;

	
	/* Initialize energy, strength, field values */
	magnet_energy_set_point=excitation_set_point=NAN;
	energy=0.1;
        min_me=0.0;
        max_me=0.0;
        min_kick=0.0;
        max_kick=0.0;
	first_read=true;
	read_stamp.tv_sec=0;
	read_stamp.tv_usec=0;
	//--------------------------------------------
	get_device_property();
	//--------------------------------------------
	//--------------------------------------------
	if(magnetType == "")
	{
		ERROR_STREAM<<" The device property MagnetType is not set" << std::endl;
		exit(EXIT_FAILURE);
	}
	if(powerSupply == "")
	{
		ERROR_STREAM<<" The device property PowerSupply is not set" << std::endl;
		exit(EXIT_FAILURE);
	}
	if(magnetName == "")
	{
		ERROR_STREAM<<" The device property magnetName is not set" << std::endl;
		exit(EXIT_FAILURE);
	}
	
	{
		try
		{
			if(dbUser != "" && dbHost != "" && dbPassword != "")
			{
				DEBUG_STREAM<< "Using the user, host and password found in the  properties" << std::endl;
				mf = new Magnetfactory(dbUser.c_str(), dbPassword.c_str(), dbHost.c_str());
			}
			else
			{
				DEBUG_STREAM<< "Using the user, host and password from MAGNET_DB env. var." << std::endl;
				mf = new Magnetfactory();
			}
			
		}
		catch(std::invalid_argument& ia)
		{
			Tango::Except::throw_exception(ia.what(), "Error creating the Magnetfactory",
			                               " TMagbet()::new Magnetfactory()", Tango::ERR);
			
			exit(EXIT_FAILURE);
		}
	}
	magnet = CreateMagnet(magnetType);
	if(magnet == NULL)
	{
		ERROR_STREAM<<"Error creating magnet object. Exiting"<<std::endl;
		exit(EXIT_FAILURE);
	}
	
	try
	{
		dev = new Tango::DeviceProxy(powerSupply);
                dev->set_transparency_reconnection(true);
	}
	catch(Tango::DevFailed &e)
	{
		ERROR_STREAM<<"Error connecting to device. Exiting" << std::endl;
		Tango::Except::print_exception(e);
		exit(EXIT_FAILURE);
	}
        magnet->getExcitationRange(min_ex,max_ex);
        /* read the current limits */
	try
	{
	    Tango::AttributeInfoEx atinfo;
	    atinfo = dev->get_attribute_config(pSExcitationAttributeName);
	    
	    min_ps=atof(atinfo.min_value.c_str());
	    max_ps=atof(atinfo.max_value.c_str());
	    DEBUG_STREAM << "min_ps:" << min_ps << "  max_ps:" << max_ps << std::endl;
	    if(min_ex < min_ps){ 
		min_ex = min_ps;
	    }
	    if(max_ex > max_ps){
		max_ex = max_ps;
	    }
	}
	catch(Tango::DevFailed &e)
	{
                min_ps=min_ex;
                max_ps=max_ex;
		ERROR_STREAM<<"error reading attribute config from : " << powerSupply.c_str()<<std::endl;
// 		Tango::Except::re_throw_exception(e, "Error reading attribute config from the Tango device", 
// 		                                  "get_attribute_config error", "getProperty()",
// 		                                  Tango::ERR);
	}
        
        DEBUG_STREAM << "min_ex:" << min_ex << "  max_ex:" << max_ex << std::endl;
        try{
            min_me=magnet->excitationToEnergy(min_ex);
            max_me=magnet->excitationToEnergy(max_ex);
            DEBUG_STREAM << "min_me:" << min_me << "  max_me:" << max_me << std::endl;
        }
        catch(std::range_error& e)
        {
            Tango::Except::throw_exception(e.what(), "Error reading strength", 
                                           "read_Strength(): excitationToStrength(excitation, energy)", 
                                           Tango::ERR);
            return;
        }
	
	set_state(Tango::INIT);
	//create Wattributes for updating write part
	Tango::MultiAttribute *attr_list = get_device_attr();
	attr_magnet_energy=&(attr_list->get_w_attr_by_name ("MagnetEnergy"));
	attr_excitation=&(attr_list->get_w_attr_by_name ("Excitation"));
	attr_energy=&(attr_list->get_w_attr_by_name ("Energy"));
	attr_kick=&(attr_list->get_w_attr_by_name ("Kick"));
        
        //make final init by reading
        setStrengthLimits();
        try{
		GetExcitationFromPowerSupply();
 		updateInternalValues();
		
	}
	catch(Tango::DevFailed &e)
        {
            set_state(Tango::FAULT);
            set_status("Error reading power supply settings");
            ERROR_STREAM<<"Error reading power supply settings from : " << powerSupply.c_str()<<std::endl;
            Tango::Except::print_exception(e);
        }
        catch(std::range_error& e)
        {
            set_state(Tango::FAULT);
            set_status("Error reading the magnet parameters from the excitation (range_error)");
            ERROR_STREAM << "Error reading the magnet parameters from the excitation (range_error)"<<std::endl;
        }
        

}


//+----------------------------------------------------------------------------
//
// method : 		ABending::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void ABending::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------
	
	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("DbHost"));
	dev_prop.push_back(Tango::DbDatum("DbPassword"));
	dev_prop.push_back(Tango::DbDatum("DbUser"));
	dev_prop.push_back(Tango::DbDatum("MagnetName"));
	dev_prop.push_back(Tango::DbDatum("MagnetType"));
	dev_prop.push_back(Tango::DbDatum("PowerSupply"));
	dev_prop.push_back(Tango::DbDatum("PSExcitationAttributeName"));
	dev_prop.push_back(Tango::DbDatum("ReadInterval"));
	dev_prop.push_back(Tango::DbDatum("MadSignConvention"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	ABendingClass	*ds_class =
		(static_cast<ABendingClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize DbHost from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  dbHost;
	else {
		//	Try to initialize DbHost from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  dbHost;
	}
	//	And try to extract DbHost value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dbHost;

	//	Try to initialize DbPassword from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  dbPassword;
	else {
		//	Try to initialize DbPassword from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  dbPassword;
	}
	//	And try to extract DbPassword value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dbPassword;

	//	Try to initialize DbUser from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  dbUser;
	else {
		//	Try to initialize DbUser from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  dbUser;
	}
	//	And try to extract DbUser value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dbUser;

	//	Try to initialize MagnetName from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  magnetName;
	else {
		//	Try to initialize MagnetName from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  magnetName;
	}
	//	And try to extract MagnetName value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  magnetName;

	//	Try to initialize MagnetType from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  magnetType;
	else {
		//	Try to initialize MagnetType from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  magnetType;
	}
	//	And try to extract MagnetType value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  magnetType;

	//	Try to initialize PowerSupply from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  powerSupply;
	else {
		//	Try to initialize PowerSupply from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  powerSupply;
	}
	//	And try to extract PowerSupply value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  powerSupply;

	//	Try to initialize PSExcitationAttributeName from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  pSExcitationAttributeName;
	else {
		//	Try to initialize PSExcitationAttributeName from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  pSExcitationAttributeName;
	}
	//	And try to extract PSExcitationAttributeName value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  pSExcitationAttributeName;

	//	Try to initialize ReadInterval from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  readInterval;
	else {
		//	Try to initialize ReadInterval from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  readInterval;
	}
	//	And try to extract ReadInterval value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  readInterval;

	//	Try to initialize MadSignConvention from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  madSignConvention;
	else {
		//	Try to initialize MadSignConvention from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  madSignConvention;
	}
	//	And try to extract MadSignConvention value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  madSignConvention;



	//	End of Automatic code generation
	//------------------------------------------------------------------
	
}
//+----------------------------------------------------------------------------
//
// method : 		ABending::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void ABending::always_executed_hook()
{
	//read_and_update();
}
//+----------------------------------------------------------------------------
//
// method : 		ABending::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void ABending::read_attr_hardware(std::vector<long> &attr_list)
{
	DEBUG_STREAM << "ABending::read_attr_hardware(std::vector<long> &attr_list) entering... "<< std::endl;
	//	Add your own code here
	read_and_update();
}
//+----------------------------------------------------------------------------
//
// method : 		ABending::read_ExcitationSetPoint
// 
// description : 	Extract real attribute values for ExcitationSetPoint acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_ExcitationSetPoint(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_ExcitationSetPoint(Tango::Attribute &attr) entering... "<< std::endl;
	attr.set_value(attr_ExcitationSetPoint_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MagnetEnergySetPoint
// 
// description : 	Extract real attribute values for MagnetEnergySetPoint acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MagnetEnergySetPoint(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MagnetEnergySetPoint(Tango::Attribute &attr) entering... "<< std::endl;
	attr.set_value(attr_MagnetEnergySetPoint_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_KickSetPoint
// 
// description : 	Extract real attribute values for KickSetPoint acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_KickSetPoint(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_KickSetPoint(Tango::Attribute &attr) entering... "<< std::endl;
	attr.set_value(attr_KickSetPoint_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MinMagnetEnergy
// 
// description : 	Extract real attribute values for MinMagnetEnergy acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MinMagnetEnergy(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MinMagnetEnergy(Tango::Attribute &attr) entering... "<< std::endl;
        attr.set_value(attr_MinMagnetEnergy_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MaxMagnetEnergy
// 
// description : 	Extract real attribute values for MaxMagnetEnergy acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MaxMagnetEnergy(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MaxMagnetEnergy(Tango::Attribute &attr) entering... "<< std::endl;
        attr.set_value(attr_MaxMagnetEnergy_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MinKick
// 
// description : 	Extract real attribute values for MinKick acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MinKick(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MinKick(Tango::Attribute &attr) entering... "<< std::endl;
        attr.set_value(attr_MinKick_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MaxKick
// 
// description : 	Extract real attribute values for MaxKick acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MaxKick(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MaxKick(Tango::Attribute &attr) entering... "<< std::endl;
        attr.set_value(attr_MaxKick_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_Energy
// 
// description : 	Extract real attribute values for Energy acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_Energy(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_Energy(Tango::Attribute &attr) entering... "<< std::endl;
	*attr_Energy_read = energy;
	attr.set_value(attr_Energy_read);
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::write_Energy
// 
// description : 	Write Energy attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ABending::write_Energy(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ABending::write_Energy(Tango::WAttribute &attr) entering... "<< std::endl;
	double valold=attr_Energy_write;
	attr.get_write_value(attr_Energy_write);
	if( std::isnan(attr_Energy_write) || std::isinf(attr_Energy_write) ){
		attr_Energy_write=valold;
		INFO_STREAM << "ABending::write_Energy(Tango::WAttribute &attr): energy not a valid number"<< std::endl;
		Tango::Except::throw_exception((const char*) "Error: energy not a valid number ","failed!", "ABending::write_Energy()");
	}
	energy = attr_Energy_write;
	attr_kick->set_write_value(magnet->excitationToStrength(attr_Excitation_write,energy)*1000.0);
        setStrengthLimits();
	return;
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::read_Kick
// 
// description : 	Extract real attribute values for Kick acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_Kick(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_Kick(Tango::Attribute &attr) entering... "<< std::endl;
        attr.set_value(attr_Kick_read);
        return;
	
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::write_Kick
// 
// description : 	Write Kick attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ABending::write_Kick(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ABending::write_Kick(Tango::WAttribute &attr) entering... "<< std::endl;
	if((magnetType =="DIPOLE" || magnetType=="KICKER")){
		double valold=attr_Kick_write;
		attr.get_write_value(attr_Kick_write);
		
		if( std::isnan(attr_Kick_write) || std::isinf(attr_Kick_write) ){
			attr_Kick_write=valold;
			INFO_STREAM << "ABending::write_Kick(Tango::WAttribute &attr): kick not a valid number"<< std::endl;
			Tango::Except::throw_exception((const char*) "Error: kick not a valid number ","failed!", "ABending::write_Kick()");
		}
		
		double attr_real_kick_write=attr_Kick_write/1000.0; //in milliradians
		
		try{
	/* With the strength read we have to:
	 * 1. calculate the current:
	 */
			attr_Excitation_write = magnet->strengthToExcitation(attr_real_kick_write, energy);
			
	/* 2. Write the excitation to the power supply */
			SetExcitationOnPowerSupply(attr_Excitation_write);
			
	/* 3. Update other values */
			DEBUG_STREAM << "ABending new strength: " << magnet->excitationToEnergy(attr_Excitation_write);
			attr_magnet_energy->set_write_value(magnet->excitationToEnergy(attr_Excitation_write));
			attr_kick->set_write_value(magnet->excitationToStrength(attr_Excitation_write,energy)*1000.0);
			
		}
		catch(std::range_error& re)
		{
			Tango::Except::throw_exception(re.what(), 
			                               "Error getting the kick from the excitation: value out of range.",
			                               "ABending::write_Kick()::strengthToExcitation(strength, energy)",
			                               Tango::ERR);
			return;
		}
		catch(Tango::DevFailed &e)
		{
			Tango::Except::re_throw_exception(e, "Error setting excitation to the power supply",
			                                  "SetExcitationToPowerSupply(excitation) failed!", "ABending::write_Strength()",
			                                  Tango::ERR);
			return;
		}
		
	}
	else{
		double zero=0.0;
		attr_kick->set_write_value(zero);
		INFO_STREAM << "ABending::write_Kick(Tango::WAttribute &attr) force to 0.0 "<< std::endl;
		Tango::Except::throw_exception((const char*) "Error ","failed!", "ABending::write_Strength()");
	}
	return;
}



//+----------------------------------------------------------------------------
//
// method : 		ABending::read_Excitation
// 
// description : 	Extract real attribute values for Excitation acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_Excitation(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_Excitation(Tango::Attribute &attr) entering... "<< std::endl;
	attr.set_value(attr_Excitation_read);
	return;
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::write_Excitation
// 
// description : 	Write Excitation attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ABending::write_Excitation(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ABending::write_Excitation(Tango::WAttribute &attr) entering... "<< std::endl;
	double valold=attr_Excitation_write;
	attr.get_write_value(attr_Excitation_write);
	if( std::isnan(attr_Excitation_write) || std::isinf(attr_Excitation_write) ){
		attr_Excitation_write=valold;
		INFO_STREAM << "ABending::write_Excitation(Tango::WAttribute &attr): excitation not a valid number"<< std::endl;
		Tango::Except::throw_exception((const char*) "Error: excitation not a valid number ","failed!", "ABending::write_Excitation()");
	}
	/* 2. Write the excitation to the power supply */
	try
	{
		SetExcitationOnPowerSupply(attr_Excitation_write);
		attr_magnet_energy->set_write_value(magnet->excitationToEnergy(attr_Excitation_write));
		attr_kick->set_write_value(magnet->excitationToStrength(attr_Excitation_write,energy)*1000.0);
	}
	catch(Tango::DevFailed &e)
	{
		Tango::Except::re_throw_exception(e, "Error setting excitation to the power supply",
		                                  "SetExcitationToPowerSupply(excitation) failed!", "ABending::write_Excitation()",
		                                  Tango::ERR);
		return;
	}
	catch(std::range_error& e)
	{
		Tango::Except::throw_exception(e.what(), "Error reading the magnet parameters from the excitation",
		                               "write_excitation()", Tango::ERR);
		return;
	}
	return;
}



//+----------------------------------------------------------------------------
//
// method : 		ABending::read_MagnetEnergy
// 
// description : 	Extract real attribute values for MagnetEnergy acquisition result.
//
//-----------------------------------------------------------------------------
void ABending::read_MagnetEnergy(Tango::Attribute &attr)
{
	DEBUG_STREAM << "ABending::read_MagnetEnergy(Tango::Attribute &attr) entering... "<< std::endl;
// 	try{
// 		magnet_energy_set_point = magnet->excitationToEnergy(excitation_set_point);
// 	}
// 	catch(std::range_error e)
// 	{
// 		Tango::Except::throw_exception(e.what(), "Error reading the field from the excitation",
// 		                               "read_Field(): magnet->excitationToField(excitation)", Tango::ERR);
// 		return;
// 	}
// 	
// 	*attr_MagnetEnergy_read = magnet_energy_read;
	attr.set_value(attr_MagnetEnergy_read);
	return;
}

//+----------------------------------------------------------------------------
//
// method : 		ABending::write_MagnetEnergy
// 
// description : 	Write MagnetEnergy attribute values to hardware.
//
//-----------------------------------------------------------------------------
void ABending::write_MagnetEnergy(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "ABending::write_MagnetEnergy(Tango::WAttribute &attr) entering... "<< std::endl;
	
	double valold=attr_MagnetEnergy_write;
	attr.get_write_value(attr_MagnetEnergy_write);
	
	if( std::isnan(attr_MagnetEnergy_write) || std::isinf(attr_MagnetEnergy_write) ){
		attr_MagnetEnergy_write=valold;
		INFO_STREAM << "ABending::write_MagnetEnergy(Tango::WAttribute &attr): energy not a valid number"<< std::endl;
		Tango::Except::throw_exception((const char*) "Error: energy not a valid number ","failed!", "ABending::write_MagnetEnergy()");
	}
	
	
	
	try{
	/* With the strength read we have to:
	 * 1. calculate the current:
	 */
		attr_Excitation_write = magnet->energyToExcitation(attr_MagnetEnergy_write);
		
	/* 2. Write the excitation to the power supply */
		SetExcitationOnPowerSupply(attr_Excitation_write);
		
	/*3. Update Kick */
		attr_kick->set_write_value(magnet->excitationToStrength(attr_Excitation_write,energy)*1000.0);
		
	}
	catch(std::range_error& re)
	{
		Tango::Except::throw_exception(re.what(), 
		                               "Error getting the energy from the excitation: value out of range.",
		                               "ABending::write_MagnetEnergy()::energyToExcitation(energy)",
		                               Tango::ERR);
		return;
	}
	catch(Tango::DevFailed &e)
	{
		Tango::Except::re_throw_exception(e, "Error setting excitation to the power supply",
		                                  "SetExcitationToPowerSupply(excitation) failed!", "ABending::write_MagnetEnergy()",
		                                  Tango::ERR);
		return;
	}
	return;
}



int ABending::delta_milliseconds(struct timeval *t1, struct timeval *t2)
{
	__time_t sec;
	__suseconds_t usec;
	int milli;
	
	sec=t2->tv_sec - t1->tv_sec;
	if(sec>=0){
		usec=t2->tv_usec - t1->tv_usec;
		if(usec <0){
			usec+=1000000;
			sec--;
		}
		milli= sec*1000 + usec/1000;
	}
	else{
		sec=t1->tv_sec - t2->tv_sec;
		usec= t1->tv_usec -t2->tv_usec;
		if(usec <0){
			usec+=1000000;
			sec--;
		}
		milli= -(sec*1000 + usec/1000);
	}
	return milli;
}

Dipole * ABending::CreateMagnet(std::string tname)
{
	Dipole *mag = NULL;
	
	try{
		mag = mf->newDipole(magnetName.c_str());
                delete mf;
	}
	catch(std::invalid_argument& ia)
	{
		ERROR_STREAM<<"newDipole: invalid argument " << device_name<<std::endl;
		Tango::Except::throw_exception(ia.what(), "Error creating magnet",
		                               "CreateMagnet(std::string name): invalid argument", Tango::ERR);
	}
	catch(std::range_error& re)
	{
		ERROR_STREAM<<"newDipole: range error " << device_name<<std::endl;
		Tango::Except::throw_exception(re.what(), "Error creating magnet", 
		                               "CreateMagnet(std::string name): range error", Tango::ERR);
	}
	catch(...)
	{
		ERROR_STREAM<<"newDipole: unexpected exception " << device_name<<std::endl;
		Tango::Except::throw_exception("unknwon", "Error creating magnet", 
		                               "CreateMagnet(std::string name)", Tango::ERR);
	}
	if(mag) _name_of_type=tname;
	else _name_of_type="";
	if(madSignConvention){
				mag->setFocSignConvention(foc_sign_mad);
				DEBUG_STREAM<<"Using MAD sign convention for " << device_name<<std::endl;
			}else{
				mag->setFocSignConvention(foc_sign_vicky);
				DEBUG_STREAM<<"Using vicky sign convention for " << device_name<<std::endl;
			}
	return mag;
}


void ABending::SetExcitationOnPowerSupply(double ex)
{
	Tango::DeviceAttribute da(pSExcitationAttributeName, ex);
	if (!use_double_attribute){
	    float fex=ex;
	    da<<fex;
	}
	/* write the current */
	try
	{
		dev->write_attribute(da);
		attr_excitation->set_write_value(ex);
		set_state(Tango::ON);
	}
	catch(Tango::DevFailed& e)
	{
		ERROR_STREAM<<"failed to write current on the device << "<< powerSupply.c_str()<<" "<< ex<< std::endl;
		set_state(Tango::UNKNOWN);	
		Tango::Except::re_throw_exception(e, "Error writing current on the Tango device", 
		                                  "write_attribute() error", "SetExcitationOnPowerSupply(): write_attribute()",
		                                  Tango::ERR);
	}
}

void ABending::GetExcitationFromPowerSupply()
{
	
    Tango::DeviceAttribute da;
        //read the PS state
        try
        {
	    Tango::DeviceData dd;
	    dd=dev->command_inout("state");
	    dd>>powersupply_state;
	    DEBUG_STREAM<<"OK reading state : " <<powersupply_state<<std::endl;
	    set_state(powersupply_state);
	    switch (powersupply_state){
		case Tango::OFF:
		case Tango::STANDBY: set_status("magnet is off"); break;
		case Tango::RUNNING: set_status("magnet is cycling"); break;
		case Tango::MOVING:
		case Tango::ON: set_status("magnet is on"); break;
		case Tango::FAULT: set_status("power supply is in fault"); break;
		case Tango::ALARM: set_status("power supply is in alarm"); break;
		default: set_status("unforeseen status");
	    }
	}

    catch(Tango::DevFailed &e)
    {
        set_state(Tango::FAULT);
        set_status("error reading power supply state");
        ERROR_STREAM<<"error reading state  from : " << powerSupply.c_str()<<std::endl;
	return;
    }

    /* read the current */
    try
    {
        da = dev->read_attribute(pSExcitationAttributeName);
        if (da.get_type() == Tango::DEV_DOUBLE) { //CS. now handles also weird power supplies whit Float attribute -- C.S. 2010-9-10
            std::vector<double> outval;
            da >> outval;
	    excitation_read=outval[0]; //get the reading
            excitation_set_point=outval[1];        // get the setting
            attr_Excitation_write=outval[1];  // N.B. get the write or setting part of the attribute!!
	    use_double_attribute=true;
        }
        if (da.get_type() ==Tango::DEV_FLOAT){
             std::vector<float> outval;
            da >> outval;
	    excitation_read=outval[0];  //get the reading
            excitation_set_point=outval[1]; // get the setting
            attr_Excitation_write=outval[1]; // N.B. get the write or setting part of the attribute!!
	    use_double_attribute=false;
        }
        attr_excitation->set_write_value(attr_Excitation_write);
    }
    catch(Tango::DevFailed &e)
    {
        set_state(Tango::FAULT);
	set_status("error reading power supply current");
        ERROR_STREAM<<"error reading current from : " << powerSupply.c_str()<<std::endl;
    }
    return;
	
}




void ABending::updateInternalValues(void)
{
    DEBUG_STREAM << "updateInternalValues() : Exc = "<<attr_Excitation_write<<std::endl;
    try{
        magnet_energy_set_point=magnet->excitationToEnergy(attr_Excitation_write);
        magnet_energy_read= magnet->excitationToEnergy(excitation_read);
        attr_magnet_energy->set_write_value(magnet_energy_set_point);
    }
    catch(std::range_error& e)
    {
        if (powersupply_state ==  Tango::OFF || powersupply_state == Tango::FAULT || powersupply_state == Tango::STANDBY ){
            magnet_energy_set_point=0.0;
            magnet_energy_read=0.0;
            attr_magnet_energy->set_write_value(magnet_energy_set_point);
            set_status("PS OFF,STANDBY or FAULT : energy=0");
        }
        else{
            magnet_energy_set_point=NAN;
            magnet_energy_read=NAN;
            set_state(Tango::FAULT);
            set_status("error converting current to energy");
        }
    }
    //attr_energy->set_write_value(energy);
    try{
        kick_set_point=magnet->excitationToStrength(attr_Excitation_write,energy)*1000.0;
        kick_read=magnet->excitationToStrength(excitation_read,energy)*1000.0;
        attr_kick->set_write_value(kick_set_point); //in milliradians
    }
    catch(std::range_error& e)
    {
        if (powersupply_state ==  Tango::OFF || powersupply_state == Tango::FAULT || powersupply_state == Tango::STANDBY){
            kick_set_point=0.0;
            kick_read=0.0;
            attr_kick->set_write_value(kick_set_point); //in milliradians
        }
        else{
            kick_set_point=NAN;
            kick_read=NAN;
            set_state(Tango::FAULT);
            set_status("error converting current to kick");
        }
    }
}


void ABending::read_and_update(void)
{
        struct timeval now;
	gettimeofday(&now,0);
	unsigned long delta=delta_milliseconds(&read_stamp,&now);
	if(delta <= readInterval) return;
	{
		try{
			GetExcitationFromPowerSupply();
			gettimeofday(&read_stamp,0); //record read time
			updateInternalValues();
			if(first_read){
				first_read=false;
			}
			
		}
		catch(std::range_error& e)
		{
// 			Tango::Except::throw_exception(e.what(), "Error reading the magnet parameters from the excitation",
// 			                               "read_attr_hardware()", Tango::ERR);
			set_state(Tango::FAULT);
			set_status("error converting current field values");
			return;
		}
	}
}

 void ABending::setStrengthLimits(void){
     try{
        min_kick=magnet->excitationToStrength(min_ex,energy)*1000.0;
        max_kick=magnet->excitationToStrength(max_ex,energy)*1000.0;
    }
    catch(std::range_error& e)
    {
        Tango::Except::throw_exception(e.what(), "Error reading strength", 
                                       "read_Strength(): excitationToStrength(excitation, energy)", 
                                       Tango::ERR);
        return;
    }
 }






}	//	namespace
