//+=============================================================================
//
// file :        main.cpp
//
// description : C++ source for a TANGO device server main.
//               The main rule is to initialise (and create) the Tango
//               system and to create the DServerClass singleton.
//               The main should be the same for every Tango device server.
//
// project :     TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.3 $ $
//
// $Log: main.cpp,v $
// Revision 1.3  2019-01-14 16:12:42  claudio
// #6723: 0 field when OFF,FAULT,STANDBY
//
// Revision 1.2  2010-08-27 13:20:36  claudio
// revision std::string
//
// Revision 1.1  2010/02/23 15:45:19  claudio
// bending magnet model - first import
//
// Revision 1.1  2007/08/06 14:37:09  claudio
// *** empty log message ***
//
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================

#include <tango.h>
#define CVSVERSION "$Name:  $"

int main(int argc,char *argv[])
{
	const char *cvs_version = CVSVERSION;
	Tango::Util *tg = nullptr;
	try
	{
		// Initialise the device server
		//----------------------------------------
		tg = Tango::Util::init(argc,argv);

		// Create the device server singleton 
		//	which will create everything
		//----------------------------------------
		tg->server_init(false);

		// Run the std::endless loop
		//----------------------------------------
		std::cout << "Ready to accept request" << std::endl;
		tg->server_run();
	}
	catch (std::bad_alloc&)
	{
		std::cout << "Can't allocate memory to store device object !!!" << std::endl;
		std::cout << "Exiting" << std::endl;
	}
	catch (CORBA::Exception &e)
	{
		Tango::Except::print_exception(e);
		
		std::cout << "Received a CORBA_Exception" << std::endl;
        std::cout << "Exiting " << cvs_version << std::endl;
	}
	tg->server_cleanup();
	return(0);
}
